#!/usr/bin/env python
################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
################################################################################
# glite-wn-config v. 2.3.7
#
# Post-installation script for configuring the gLite Standard Worker Node
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
# Diana Bosio <Diana.Bosio@cern.ch>
# Leanne Guy <leanne.guy@cern.ch>
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id$
#
# Usage: python glite-wn-config [-c|-v|-h|--help]
#        -c                  print configuration
#        -v                  print version
#        -h,--help            print usage info
#        --configure         print usage info
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
################################################################################

import os,string,getopt
import sys, posix

from gLiteInstallerLib import gLib 
from gLiteInstallerLib import ConfigParams
from gridSecurity import gridSecurity
from serviceDiscovery import ServiceDiscovery

# Set global variables here 
global params                         # all config values from the XML file

class glite_wn:

    #################################################################
    # init 
    #
    # init
    #
    #################################################################        
    def __init__(self):
        self.verbose = 0
        self.version = "2.3.7"
        self.name = "glite-wn"
        self.friendly_name = "gLite Worker Node"


    #################################################################
    # banner 
    #
    # print banner
    #
    #################################################################        
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    

    #################################################################
    # copyright 
    #
    # print copyright
    #
    #################################################################    
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # showVersion
    #
    # print version
    #
    #################################################################    
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    # param  message   extra message to be printed
    #
    #################################################################    
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % os.environ['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf      print the service configuration'
        print '    -v, --version        print the version of the configuration script'
        print '    -h, --help           print this usage information'
        print '    --configure          configure the WN components'
        print ''
        print '    --siteconfig=URL    URL of the site configuration file'
        print ''
        print '\n'


    #################################################################
    # configure
    #
    # configure WN
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):
        
        print "Configuring security ...                                 "
        errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])

        if errorcode:
            print "Configuring security                                      ",
            glib.printFailedMessage()
            return 1
    
        print "Configuring security                                      ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------
    
        print "\nThis WN will be configured to serve the following VOs:"
        for vo in glib.getVoList():
            print "=> %s" % vo
            
        #-------------------------------------------------------------
        # Configure pool accounts
        #-------------------------------------------------------------
        
        if os.geteuid() == 0:
            ret_val = gsec.create_pool_accounts()
            if ret_val == 1:
                return 1
    
        # ------------------------------------------------------------
        # configure service discovery for Fireman catalog
        # ------------------------------------------------------------

        errorcode = serviceDiscovery.configure(params,glib)
        if errorcode:
            return 1
        
        # ------------------------------------------------------------
        # Configure subservices
        # ------------------------------------------------------------
    
        serviceList = params['wn.ServiceList']
        #serviceList = params['wn.ServiceList'].split(',')
        
        for item in serviceList:
            if not os.path.exists("%s/%s-config.py" % (glib.getScriptPath(),item.strip())):
                glib.printErrorMessage("The %s/%s-config.py script cannot be found" % (glib.getScriptPath(),item.strip()))
                return 1
            
        for item in serviceList:
            if item != "glite-dgas-client":
			cmd = "python %s/%s-config.py --configure --subservice" % (glib.getScriptPath(),item.strip())
			if os.system(cmd):
				glib.printErrorMessage("Failed to configure %s" % item.strip())
				return 1
            
        # Protect env script
        glib.protectEnvFile()
    
        return 0

    #################################################################
    # createParameterNode
    #
    # create a parameter node in xml dom tree
    #
    # param   factory   dom tree document
    #         name      name of node
    #         value     value of node
    #
    # return node
    #
    #################################################################
    def createParameterNode(self, factory, name, value):
        node = factory.createElement(name)
        nodeText = factory.createTextNode(value)
        node.appendChild(nodeText)

        return node
        
#-------------------------------------------------------------------------------
# Init params
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    params['custom.environment'] = []

#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment file will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        if os.path.exists(params['GLITE_LOCATION']):
            if not os.access(params['GLITE_LOCATION'], os.W_OK):
                print "    GLITE_LOCATION is set to the non-writable directory %s -" % params['GLITE_LOCATION'],
                params['GLITE_LOCATION'] = "%s" % (installPrefix)
                print "it will be changed to %s" % params['GLITE_LOCATION']
            else:
                if not params['GLITE_LOCATION'] == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        if os.path.exists(params['ca.certificates.dir']):
            if not os.access('ca.certificates.dir', os.W_OK):
                print "    ca.certificates.dir is set to the non-writable directory %s - " % params['ca.certificates.dir'],
                params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
                print "it will be changed to %s" % params['ca.certificates.dir']
            else:
                if not params['ca.certificates.dir'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
        
        print "\n"

    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    glib.export('GLITE_LOCATION_TMP');
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.export('GLOBUS_LOCATION');
    glib.export('GPT_LOCATION');
    glib.export('JAVA_HOME');

    # bin and lib paths
    glib.addEnvPath("PATH","/usr/bin")
    glib.addEnvPath("PATH","$JAVA_HOME/bin")
    glib.addEnvPath("PATH","$GLOBUS_LOCATION/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/externals/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","/usr/local/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/externals/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")

    # Perl
    glib.addEnvPath("PERL5LIB", "$GPT_LOCATION/lib/perl")
    glib.addEnvPath("PERL5LIB", "$GLITE_LOCATION/lib/perl5")

    # MANPATH
    glib.addEnvPath('MANPATH', '$GLOBUS_LOCATION/man')
    glib.addEnvPath('MANPATH', '$GLITE_LOCATION/share/man')

    # Add custom entries
    for entry in params['custom.environment']:
        if entry.strip() != "":
            glib.export(entry.split(' ',1)[0],'"' + entry.split(' ',1)[1] + '"');
    
    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-wn.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)

    if not os.geteuid() == 0:
        params['glite.installer.basedir'] = glib.getScriptPath() + "../../../"
    
    # ------------------------------------------------------------
    # Set verbosity
    # ------------------------------------------------------------
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # ------------------------------------------------------------
    # Set up the environment
    # ------------------------------------------------------------
    set_env()
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # ------------------------------------------------------------
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_wn()
    service.verbose = verbose

    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
        
    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','configure','siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------
    for o, a in opts:

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":
        
            # Print configuration parameters
            if verbose:
                glib.print_params(params)
        
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Configure the service
            if service.configure() == 0:
                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" % service.friendly_name
                sys.exit(1)

            sys.exit(0)

    # ------------------------------------------------------------
    # No option given
    # ------------------------------------------------------------
    service.usage()
    sys.exit(0)
